from ms_imputedhours_core.agreements import Agreements
from ms_imputedhours_core.employee import calculate_real_capacity, get_real_fte
from ms_imputedhours_core.employee.data.bigquery import (
    get_all_data,
    get_all_data_by_dates,
    get_all_employee_capacity,
    get_imputed_hours,
    get_out_date_imputations,
    get_successfactor_all_data,
)
from ms_imputedhours_core.employee.data.helpers.alert import (
    should_exclude_employee,
)
from ms_imputedhours_core.employee.data.transformers.hours import (
    group_task_by_email,
    group_task_by_outdated,
    transform_all_capacities,
    transform_successfactor_all_data,
)
from ms_imputedhours_core.office import get_real_office_name

AGREEMENTS_SPREADSHEET_ID = '1gMQ30Sl0QHhocTQgbYpWvS6DarhcRIeBIANW8FYLTTQ'
AWAIT_FOR_GOOLE_API_BACK_PRESSURE = 5
EMPTY_SUCCESSFACTOR_DATA = {
    'office_name': '',
    'FTE': 0,
    'hiring_date': None,
    'enddate': None,
}
EMPTY_AGREEMENT_HOURS_DATA = {'totalHours': 0.0, 'days': {}}
EMPTY_EMPLOYEE_DATA = {
    'data': {},
    'yearSelected': 0,
    'monthSelected': 0,
    'agreementHours': 0,
    'successfactor_data': EMPTY_SUCCESSFACTOR_DATA,
}
GOOGLE_API_MAX_REQUESTS_PER_SEG = 60


def get_all_employees_monthly_data_by_office(date, office_name):
    data = {}
    agreement_hours = Agreements(AGREEMENTS_SPREADSHEET_ID).get_hours_by_month(
        date.month, date.year, office_name
    )
    successfactor_all_data = transform_successfactor_all_data(
        get_successfactor_all_data(office_name)
    )
    if successfactor_all_data:
        office_emails = successfactor_all_data.keys()
        current_month_data = group_task_by_email(
            get_all_data(date, office_emails)
        )
        all_employee_ftes = transform_all_capacities(
            get_all_employee_capacity(date.month, date.year)
        )

        for email in office_emails:
            successfactor_data = successfactor_all_data.get(
                email, EMPTY_SUCCESSFACTOR_DATA
            )

            if should_exclude_employee(
                email,
                date,
                successfactor_data['hiring_date'],
                successfactor_data['enddate'],
            ):
                continue

            office_name = get_real_office_name(
                successfactor_data.get('office_name')
            )

            fte = get_real_fte(
                successfactor_data, all_employee_ftes.get(email), date
            )

            real_capacity = calculate_real_capacity(
                agreement_hours,
                successfactor_data.get('hiring_date'),
                successfactor_data.get('enddate'),
                date,
                date,
                fte,
            )

            # Updating fte value
            successfactor_data['FTE'] = fte

            if email not in data.keys():
                data[email] = {
                    'data': {
                        'totalTimeSpent': current_month_data.get(
                            email, {}
                        ).get('totalHours', 0)
                    },
                    'agreementHours': {
                        'totalHours': agreement_hours.get('totalHours', 0),
                        'realcapacity': real_capacity,
                    },
                    'successfactor_data': successfactor_data,
                    'yearSelected': date.year,
                    'monthSelected': date.month,
                }

    return data


def get_all_employee_data_by_range(from_date, to_date, office_name):
    employees_data = {}
    agreement_hours = Agreements(AGREEMENTS_SPREADSHEET_ID).get_hours_by_range(
        from_date, to_date, office_name
    )
    successfactor_all_data = transform_successfactor_all_data(
        get_successfactor_all_data(office_name)
    )
    office_emails = successfactor_all_data.keys()
    current_month_data = group_task_by_email(
        get_all_data_by_dates(from_date, to_date, office_emails)
    )
    all_employee_ftes = transform_all_capacities(
        get_all_employee_capacity(from_date.month, from_date.year)
    )

    for email in office_emails:
        successfactor_data = successfactor_all_data.get(
            email, EMPTY_SUCCESSFACTOR_DATA
        )

        if should_exclude_employee(
            email,
            from_date,
            successfactor_data['hiring_date'],
            successfactor_data['enddate'],
        ):
            continue

        employee_fte = get_real_fte(
            successfactor_data, all_employee_ftes.get(email), from_date
        )
        employee_current_capacity = (
            current_month_data.get(email, EMPTY_AGREEMENT_HOURS_DATA)[
                'totalHours'
            ]
            / 3600
        )
        employee_real_capacity = calculate_real_capacity(
            agreement_hours,
            successfactor_data.get('hiring_date'),
            successfactor_data.get('enddate'),
            from_date,
            to_date,
            employee_fte,
            calculate_range=True,
        )

        capacity_percentage = 0
        if employee_real_capacity == 0:
            capacity_percentage = 100
        else:
            capacity_percentage = round(
                (employee_current_capacity * 100) / employee_real_capacity, 2
            )

        employees_data[email] = {
            'real_capacity': employee_real_capacity,
            'current_capacity': employee_current_capacity,
            'current_percentage_hours_imputed': capacity_percentage,
            'supervisor': successfactor_data['supervisor'],
            'name': successfactor_data['name'],
        }

    return employees_data


def get_all_imputations_per_day(from_date, to_date):
    current_month_data = group_task_by_email(
        get_imputed_hours(from_date, to_date)
    )

    return current_month_data


def get_all_out_date_imputations(first_day, last_day, office_name):
    successfactor_all_data = transform_successfactor_all_data(
        get_successfactor_all_data(office_name)
    )
    current_data = group_task_by_outdated(
        get_out_date_imputations(first_day, last_day)
    )
    employees_data = {}
    for email in successfactor_all_data.keys():
        successfactor_data = successfactor_all_data.get(
            email, EMPTY_SUCCESSFACTOR_DATA
        )

        if should_exclude_employee(
            email,
            first_day,
            successfactor_data['hiring_date'],
            successfactor_data['enddate'],
        ):
            continue

        if current_data.get(email):
            supervisor_email = successfactor_data['supervisor']
            if supervisor_email not in employees_data.keys():
                employees_data[supervisor_email] = {}

            employees_data[supervisor_email][email] = {
                'data': current_data[email],
                'name': successfactor_data['name'],
            }

    return employees_data



import unittest

from ms_imputedhours_core.office import get_real_office_name


class TestSuite(unittest.TestCase):
    def test_get_real_office_name_return_same_param_when_office_name_is_not_mapped(
        self,
    ):  # noqa: E501
        expected_result = 'Valencia'
        office_name = 'Valencia'

        result = get_real_office_name(office_name)

        self.assertEqual(result, expected_result)

    def test_get_real_office_name_return_real_office_name_when_is_mapped(self):
        expected_result = 'Madrid - Ingeniería'
        office_name = 'Madrid - Ing'

        result = get_real_office_name(office_name)

        self.assertEqual(result, expected_result)



import unittest
from datetime import date, datetime
from unittest.mock import Mock, patch

from freezegun import freeze_time

from ms_imputedhours_core.employee.data import get_all_employee_data_by_range

DEFAULT_AGREEMENT_HOURS = {
    'totalHours': 142.0,
    'days': {
        '1-4': '0',
        '2-4': '0',
        '3-4': '8',
        '4-4': '5.5',
        '5-4': '0',
        '6-4': '0',
        '7-4': '8',
        '8-4': '8',
        '9-4': '0',
        '10-4': '8',
        '11-4': '5.5',
        '12-4': '0',
        '13-4': '0',
        '14-4': '8',
        '15-4': '8',
        '16-4': '8',
        '17-4': '8',
        '18-4': '5.5',
        '19-4': '0',
        '20-4': '0',
        '21-4': '8',
        '22-4': '8',
        '23-4': '8',
        '24-4': '8',
        '25-4': '5.5',
        '26-4': '0',
        '27-4': '0',
        '28-4': '8',
        '29-4': '8',
        '30-4': '8',
    },
}


class TestSuite(unittest.TestCase):
    def _create_Agreements_mock(self, Agreements):
        get_hours_by_range_mock = Mock()
        get_hours_by_range_mock.get_hours_by_range.return_value = (
            DEFAULT_AGREEMENT_HOURS  # noqa: E501
        )

        Agreements.return_value = get_hours_by_range_mock
        return Agreements

    def _create_successfactor_all_data_mock(self, get_successfactor_all_data):
        get_successfactor_all_data.return_value = [
            {
                'email': 'employee1@makingscience.com',
                'hiringdate': date(2021, 4, 1),
                'internshipdate': None,
                'fte': '100',
                'supervisor': 'employee2@makingscience.com',
                'office': 'Madrid - Ing',
                'name': 'Employee1',
                'lastname': 'Doe',
            },
            {
                'email': 'employee2@makingscience.com',
                'hiringdate': date(2021, 4, 1),
                'internshipdate': None,
                'fte': '50',
                'supervisor': 'employee20@makingscience.com',
                'office': 'Madrid - Ing',
                'name': 'Employee2',
                'lastname': 'Doe',
            },
            {
                'email': 'employee3@makingscience.com',
                'hiringdate': date(2021, 4, 1),
                'internshipdate': date(2021, 12, 27),
                'fte': '100',
                'supervisor': 'employee2@makingscience.com',
                'office': 'Madrid - Ing',
                'name': 'Employee1',
                'lastname': 'Doe',
            },
            {
                'email': 'employee4@makingscience.com',
                'hiringdate': date(2022, 4, 1),
                'internshipdate': None,
                'fte': '50',
                'supervisor': 'employee2@makingscience.com',
                'office': 'Madrid - Ing',
                'name': 'Employee4',
                'lastname': 'Doe',
            },
        ]
        return get_successfactor_all_data

    def _create_get_all_data_by_dates_mock(self, get_all_data_by_dates):
        get_all_data_by_dates.return_value = [
            {
                'timeSpent': 28800,
                'authorEmail': 'employee1@makingscience.com',
                'started': date(2022, 4, 15),
            },
            {
                'timeSpent': 28800,
                'authorEmail': 'employee1@makingscience.com',
                'started': date(2022, 4, 16),
            },
            {
                'timeSpent': 28800,
                'authorEmail': 'employee4@makingscience.com',
                'started': date(2022, 4, 15),
            },
            {
                'timeSpent': 28800,
                'authorEmail': 'employee4@makingscience.com',
                'started': date(2022, 4, 16),
            },
        ]

        return get_all_data_by_dates

    def _create_get_all_employee_capacity_mock(
        self, get_all_employee_capacity
    ):  # noqa: E501
        get_all_employee_capacity.return_value = [
            {
                'email': 'employee1@makingscience.com',
                'fte': 1.0,
            },
            {
                'email': 'employee2@makingscience.com',
                'fte': 1.0,
            },
            {
                'email': 'employee4@makingscience.com',
                'fte': 0.5,
            },
        ]
        return get_all_employee_capacity

    @freeze_time("2022-4-11")
    @patch('ms_imputedhours_core.employee.data.get_all_employee_capacity')
    @patch('ms_imputedhours_core.employee.data.get_all_data_by_dates')
    @patch('ms_imputedhours_core.employee.data.get_successfactor_all_data')
    @patch('ms_imputedhours_core.employee.data.Agreements')
    def test_get_all_employee_data_by_range_return_employee_full_capacity(
        self,
        Agreements,
        get_successfactor_all_data,
        get_all_data_by_dates,
        get_all_employee_capacity,
    ):
        Agreements = self._create_Agreements_mock(Agreements)
        get_successfactor_all_data = self._create_successfactor_all_data_mock(
            get_successfactor_all_data
        )
        get_all_data_by_dates = self._create_get_all_data_by_dates_mock(
            get_all_data_by_dates
        )
        get_all_employee_capacity = (
            self._create_get_all_employee_capacity_mock(  # noqa: E501
                get_all_employee_capacity
            )
        )

        from_date = datetime.strptime('2022-04-1', '%Y-%m-%d').date()
        to_date = datetime.strptime('2022-04-2', '%Y-%m-%d').date()
        office_name = 'Madrid - ing'
        expected_result = {
            'employee1@makingscience.com': {
                'real_capacity': 0.0,
                'current_capacity': 16.0,
                'current_percentage_hours_imputed': 100,
                'supervisor': 'employee2@makingscience.com',
                'name': 'Employee1 Doe',
            },
            'employee2@makingscience.com': {
                'real_capacity': 0.0,
                'current_capacity': 0.0,
                'current_percentage_hours_imputed': 100,
                'supervisor': 'employee20@makingscience.com',
                'name': 'Employee2 Doe',
            },
            'employee4@makingscience.com': {
                'real_capacity': 0.0,
                'current_capacity': 16.0,
                'current_percentage_hours_imputed': 100,
                'supervisor': 'employee2@makingscience.com',
                'name': 'Employee4 Doe',
            },
        }
        result = get_all_employee_data_by_range(
            from_date,
            to_date,
            office_name,
        )

        self.assertDictEqual(result, expected_result)

    @freeze_time("2022-4-15")
    @patch('ms_imputedhours_core.employee.data.get_all_employee_capacity')
    @patch('ms_imputedhours_core.employee.data.get_all_data_by_dates')
    @patch('ms_imputedhours_core.employee.data.get_successfactor_all_data')
    @patch('ms_imputedhours_core.employee.data.Agreements')
    def test_get_all_employee_data_by_range_return_employee_correct_data_for_day_15(  # noqa: E501
        self,
        Agreements,
        get_successfactor_all_data,
        get_all_data_by_dates,
        get_all_employee_capacity,
    ):
        Agreements = self._create_Agreements_mock(Agreements)
        get_successfactor_all_data = self._create_successfactor_all_data_mock(
            get_successfactor_all_data
        )
        get_all_data_by_dates = self._create_get_all_data_by_dates_mock(
            get_all_data_by_dates
        )
        get_all_employee_capacity = (
            self._create_get_all_employee_capacity_mock(  # noqa: E501
                get_all_employee_capacity
            )
        )
        from_date = datetime.strptime('2022-04-1', '%Y-%m-%d').date()
        to_date = datetime.strptime('2022-04-15', '%Y-%m-%d').date()
        office_name = 'Madrid - ing'
        expected_result = {
            'employee1@makingscience.com': {
                'real_capacity': 59.0,
                'current_capacity': 16.0,
                'current_percentage_hours_imputed': 27.12,
                'supervisor': 'employee2@makingscience.com',
                'name': 'Employee1 Doe',
            },
            'employee2@makingscience.com': {
                'real_capacity': 59.0,
                'current_capacity': 0.0,
                'current_percentage_hours_imputed': 0.0,
                'supervisor': 'employee20@makingscience.com',
                'name': 'Employee2 Doe',
            },
            'employee4@makingscience.com': {
                'real_capacity': 29.5,
                'current_capacity': 16.0,
                'current_percentage_hours_imputed': 54.24,
                'supervisor': 'employee2@makingscience.com',
                'name': 'Employee4 Doe',
            },
        }
        result = get_all_employee_data_by_range(
            from_date,
            to_date,
            office_name,
        )

        self.assertDictEqual(result, expected_result)



import unittest
from datetime import datetime
from unittest.mock import Mock

from freezegun import freeze_time

from ms_imputedhours_core.employee import calculate_real_capacity, get_real_fte

DEFAULT_AGREEMENT_HOURS = {
    'totalHours': 150.0,
    'days': {
        1: '0',
        2: '8',
        3: '8',
        4: '5.5',
        5: '0',
        6: '0',
        7: '8',
        8: '8',
        9: '0',
        10: '8',
        11: '5.5',
        12: '0',
        13: '0',
        14: '8',
        15: '8',
        16: '8',
        17: '8',
        18: '5.5',
        19: '0',
        20: '0',
        21: '8',
        22: '8',
        23: '8',
        24: '8',
        25: '5.5',
        26: '0',
        27: '0',
        28: '8',
        29: '8',
        30: '8',
    },
}


class TestSuite(unittest.TestCase):
    @freeze_time("2022-11-11")
    def test_get_real_fte_returns_successfactor_fte_when_date_is_not_future_and_employee_has_not_fte(
        self,
    ):  # noqa: E501
        expected_result = 1.0  # 1.0 === 100%
        successfactor_data = {'FTE': 100}
        employeeFTE = {}
        date = Mock()
        date.year = 2021
        date.month = 10

        result = get_real_fte(successfactor_data, employeeFTE, date)

        self.assertEqual(result, expected_result)

    @freeze_time("2022-11-11")
    def test_get_real_fte_returns_employee_fte_when_date_is_not_future_and_employee_has_fte(
        self,
    ):  # noqa: E501
        expected_result = 0.5  # 0.5 === 50%
        successfactor_data = {'FTE': 100}
        employeeFTE = {'fte': 0.5}
        date = Mock()
        date.year = 2021
        date.month = 10

        result = get_real_fte(successfactor_data, employeeFTE, date)

        self.assertEqual(result, expected_result)

    @freeze_time("2021-11-11")
    def test_get_real_fte_returns_successfactor_fte_when_date_is_for_future(
        self,
    ):  # noqa: E501
        expected_result = 1  # 100%
        successfactor_data = {'FTE': 100}
        employeeFTE = {'fte': 50}
        date = Mock()
        date.year = 2022
        date.month = 10

        result = get_real_fte(successfactor_data, employeeFTE, date)

        self.assertEqual(result, expected_result)

    @freeze_time("2022-11-11")
    def test_calculate_real_capacity_return_full_capacity_when_is_normal_month(
        self,
    ):  # noqa: E501
        """
        A normal month is a definition for a month where the employee
        will work 100% of the hours of his agreement, and does not
        coincide with any special date as the end or start
        of the employment contract.
        """
        expected_result = 150.0
        hiring_date = datetime.strptime('17/09/2021', '%d/%m/%Y')
        end_date = None
        date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 1.0  # 100%
        calculate_range = False

        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, expected_result)

    @freeze_time("2022-11-11")
    def test_calculate_real_capacity_return_half_capacity_when_is_normal_month_and_fte_zero_point_five(
        self,
    ):  # noqa: E501
        """
        A normal month is a definition for a month where the employee
        will work 100% of the hours of his agreement, and does not
        coincide with any special date as the end or start
        of the employment contract.
        """
        expected_result = 75.0
        hiring_date = datetime.strptime('17/09/2021', '%d/%m/%Y')
        end_date = None
        date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 0.5  # 50%
        calculate_range = False

        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, expected_result)

    @freeze_time("2022-11-11")
    def test_calculate_real_capacity_return_zero_when_date_is_before_hired(
        self,
    ):  # noqa: E501
        """
        A normal month is a definition for a month where the employee
        will work 100% of the hours of his agreement, and does not
        coincide with any special date as the end or start
        of the employment contract.
        """
        expected_result = 0.0
        hiring_date = datetime.strptime('01/02/2022', '%d/%m/%Y')
        end_date = None
        date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 0.5  # 50%
        calculate_range = False

        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, expected_result)

    @freeze_time("2022-11-11")
    def test_calculate_real_capacity_return_only_work_days_when_employee_has_end_date(
        self,
    ):  # noqa: E501
        """
        A normal month is a definition for a month where the employee
        will work 100% of the hours of his agreement, and does not
        coincide with any special date as the end or start
        of the employment contract.
        """
        expected_result = 33.5
        hiring_date = datetime.strptime('01/02/2021', '%d/%m/%Y')
        end_date = datetime.strptime('15/01/2022', '%d/%m/%Y')
        date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 0.5  # 50%
        calculate_range = False

        result = calculate_real_capacity(
            DEFAULT_AGREEMENT_HOURS,
            hiring_date,
            end_date,
            date,
            to_date,
            fte,
            calculate_range,
        )

        self.assertEqual(result, expected_result)



SHEET_MAP = {
    'Madrid - Ing': 'Madrid - Ingeniería',
}


def get_real_office_name(office_name):
    if office_name in SHEET_MAP:
        office_name = SHEET_MAP[office_name]

    return office_name



import unittest
from datetime import datetime

from ..bigquery import BigqueryUpdateQueryBuilder


class TestSuite(unittest.TestCase):
    def setUp(self):
        self.email = 'test@test.com'
        self.year = 2022
        self.month = 1
        self.jira_hours = 40
        self.fte = 100
        self.office = 'Madrid'
        self.capacity = 80
        self.hiring_date = datetime.strptime(
            '02/01/22 00:00:00', '%m/%d/%y %H:%M:%S'
        )
        self.enddate = None
        self.realcapacity = 80

        self.bigquery_builder = BigqueryUpdateQueryBuilder(
            email=self.email,
            year=self.year,
            month=self.month,
            jira=self.jira_hours,
            fte=self.fte,
            office=self.office,
            capacity=self.capacity,
            hiring_date=self.hiring_date,
            enddate=self.enddate,
            realcapacity=self.realcapacity,
        )

    def test_enddate_update_query_should_return_empty_string_when_enddate_is_None(
        self,
    ):
        expected_enddate_query = ''
        self.assertEqual(
            self.bigquery_builder._set_enddate_for_update(),
            expected_enddate_query,
        )

    def test_hiring_date_update_query_should_return_date_query_when_hiring_date(
        self,
    ):
        expected_hiring_query = ", startdate='2022-02-01 00:00:00'"
        self.assertEqual(
            self.bigquery_builder._set_startdate_for_update(),
            expected_hiring_query,
        )

    def test__set_startdate_insert_should_return_startdate_value(self):
        expected_hiring_query = ",'2022-02-01 00:00:00'"
        self.assertEqual(
            self.bigquery_builder._set_startdate_insert(),
            expected_hiring_query,
        )

    def test__set_enddate_insert_should_return_enddate_value(self):
        expected_enddate_query = ""
        self.assertEqual(
            self.bigquery_builder._set_enddate_insert(), expected_enddate_query
        )

    def test_build_update_shouldd_return_update_capacity_table_query(self):
        expected_query = """
            UPDATE `ms--tiber-happeo--pro--aa82.jiradataintegration.employeecapacity` SET
                email='test@test.com',
                month='1',
                year='2022',
                fte=100,
                office='Madrid',
                jira=40,
                capacity=80,
                realcapacity=80
                , startdate='2022-02-01 00:00:00'

            WHERE
                email='test@test.com' AND
                month='1' AND
                year='2022'
        """
        expected_lines = [
            line.rstrip() for line in expected_query.splitlines()
        ]
        actual_lines = [
            line.rstrip()
            for line in self.bigquery_builder.build_update().splitlines()
        ]

        self.assertEqual(expected_lines, actual_lines)

    def test_build_insert_shouldd_return_insert_capacity_table_query_without_enddate(
        self,
    ):  # noqa: E501
        expected_query = """
            INSERT INTO `ms--tiber-happeo--pro--aa82.jiradataintegration.employeecapacity` (email, month, year, fte, office, jira, capacity, realcapacity, startdate) VALUES (
                'test@test.com',
                '1',
                '2022',
                100,
                'Madrid',
                40,
                80,
                80
                ,'2022-02-01 00:00:00'

            )
        """
        expected_lines = [
            line.rstrip() for line in expected_query.splitlines()
        ]
        actual_lines = [
            line.rstrip()
            for line in self.bigquery_builder.build_insert().splitlines()
        ]

        self.assertEqual(expected_lines, actual_lines)

    def test_build_insert_shouldd_return_insert_capacity_table_query_with_enddate(
        self,
    ):  # noqa: E501
        self.bigquery_builder = BigqueryUpdateQueryBuilder(
            email=self.email,
            year=self.year,
            month=self.month,
            jira=self.jira_hours,
            fte=self.fte,
            office=self.office,
            capacity=self.capacity,
            hiring_date=self.hiring_date,
            enddate=datetime.strptime(
                '02/01/23 00:00:00', '%m/%d/%y %H:%M:%S'
            ),
            realcapacity=self.realcapacity,
        )
        expected_query = """
            INSERT INTO `ms--tiber-happeo--pro--aa82.jiradataintegration.employeecapacity` (email, month, year, fte, office, jira, capacity, realcapacity, startdate, enddate) VALUES (
                'test@test.com',
                '1',
                '2022',
                100,
                'Madrid',
                40,
                80,
                80
                ,'2022-02-01 00:00:00'
                ,'2023-02-01 00:00:00'
            )
        """

        self.assertMultiLineEqual(
            self.bigquery_builder.build_insert(),
            expected_query,
        )

    def test_build_insert_should_return_insert_capacity_table_query(
        self,
    ):  # noqa: E501
        self.bigquery_builder = BigqueryUpdateQueryBuilder(
            email=self.email,
            year=self.year,
            month=self.month,
            jira=self.jira_hours,
            fte=self.fte,
            office=self.office,
            capacity=self.capacity,
            hiring_date=self.hiring_date,
            enddate=datetime.strptime(
                '02/01/23 00:00:00', '%m/%d/%y %H:%M:%S'
            ),
            realcapacity=self.realcapacity,
        )
        expected_query = """
            INSERT INTO `ms--tiber-happeo--pro--aa82.jiradataintegration.employeecapacity` (email, month, year, fte, office, jira, capacity, realcapacity, startdate, enddate) VALUES (
                'test@test.com',
                '1',
                '2022',
                100,
                'Madrid',
                40,
                80,
                80
                ,'2022-02-01 00:00:00'
                ,'2023-02-01 00:00:00'
            )
        """

        self.assertMultiLineEqual(
            self.bigquery_builder.build_insert(),
            expected_query,
        )



import unittest
from datetime import datetime
from unittest.mock import Mock, patch


class TestSuite(unittest.TestCase):
    def _create_GSheet_mock():
        service_mock = Mock()

        read_gsheet_mock = Mock()
        read_gsheet_mock.read_gsheet.return_value = {
            'values': [
                ['10/09/2022', '0'],
                ['11/09/2022', '0'],
                ['12/09/2022', '8'],
                ['13/09/2022', '8'],
                ['14/09/2022', '8'],
                ['15/09/2022', '8'],
                ['16/09/2022', '5.5'],
                ['17/09/2022', '0'],
                ['18/09/2022', '0'],
                ['19/09/2022', '8'],
            ]
        }
        read_gsheet_mock.get_sheetnames.return_value = {
            'sheets': [
                {'properties': {'title': 'Summary'}},
                {'properties': {'title': 'Sheet 1'}},
                {'properties': {'title': 'Sheet 2'}},
            ]
        }
        service_mock.return_value = read_gsheet_mock
        return service_mock

    @patch('ms_imputedhours_core.agreements.GSheet', new=_create_GSheet_mock())
    def test_read_gsheet_should_call_google_api_with_credentials_and_correct_params(
        self,
    ):  # noqa: E501
        expected_result = {
            'totalHours': 45.5,
            'days': {
                10: '0',
                11: '0',
                12: '8',
                13: '8',
                14: '8',
                15: '8',
                16: '5.5',
                17: '0',
                18: '0',
                19: '8',
            },
        }
        spreadsheet_id = '111111'
        sheet_name = 'SHEET_NAME_TEST'
        month = 9
        year = 2022

        from ms_imputedhours_core.agreements import Agreements

        response = Agreements(spreadsheet_id).get_hours_by_month(
            month, year, sheet_name
        )

        self.assertEqual(response, expected_result)

    @patch('ms_imputedhours_core.agreements.GSheet', new=_create_GSheet_mock())
    def test_get_hours_by_range_return_agreement_hours_by_range(self):
        expected_result = {
            'days': {
                '12-9': '8',
                '13-9': '8',
                '14-9': '8',
                '15-9': '8',
                '16-9': '5.5',
                '17-9': '0',
            },
            'totalHours': 37.5,
        }
        spreadsheet_id = '111111'
        from_date = datetime.strptime('12/09/2022', '%d/%m/%Y')
        to_date = datetime.strptime('17/09/2022', '%d/%m/%Y')
        sheet_name = 'SHEET_NAME_TEST'

        from ms_imputedhours_core.agreements import Agreements

        response = Agreements(spreadsheet_id).get_hours_by_range(
            from_date, to_date, sheet_name
        )

        self.assertEqual(response, expected_result)

    @patch('ms_imputedhours_core.agreements.GSheet', new=_create_GSheet_mock())
    def test_get_all_office_names_returns_all_sheet_names(self):
        expected_result = ['Sheet 1', 'Sheet 2']
        spreadsheet_id = '111111'

        from ms_imputedhours_core.agreements import Agreements

        response = Agreements(spreadsheet_id).get_all_office_names()

        self.assertEqual(response, expected_result)



import os

from gc_google_services_api.bigquery import execute_query

from ms_imputedhours_core.employee.data.helpers.bigquery import (
    BigqueryUpdateQueryBuilder,
)
from ms_imputedhours_core.employee.data.helpers.dates import (
    get_first_day,
    get_last_day,
)

BIGQUERY_JIRA_DATA_TYPES = os.getenv(
    'BIGQUERY_JIRA_DATA_TYPES_TABLE',
    'ms--tiber-happeo--pro--aa82.jiradataintegration.jiradatatypes',
)
BIGQUERY_SUCCESS_FACTOR = os.getenv(
    'BIGQUERY_SUCCESS_FACTOR_TABLE',
    'ms--tiber-happeo--pro--aa82.jiradataintegration.dataemployees',
)
BIGQUERY_EMPLOYEE_CAPACITY = os.getenv(
    'BIGQUERY_EMPLOYEE_CAPACITY_TABLE',
    'ms--tiber-happeo--pro--aa82.jiradataintegration.employeecapacity',
)

QUERY_HOURS_IMPUTED_OFFICE_EMPLOYEES = """
    SELECT
        *
    FROM
        `{BIGQUERY_JIRA_DATA_TYPES}`
    WHERE
        started >= "{from_date}" AND
        started <= "{to_date}" AND
        authorEmail IN ({emails})
"""

QUERY_HOURS_IMPUTED_ALL_EMPLOYEES = """
    SELECT
        *
    FROM
        `{BIGQUERY_JIRA_DATA_TYPES}`
    WHERE
        started >= "{from_date}" AND
        started <= "{to_date}"
"""

QUERY_HOURS_IMPUTED = """
    SELECT
        *
    FROM
        `{BIGQUERY_JIRA_DATA_TYPES}`
    WHERE
        started >= "{from_date}" AND
        started <= "{to_date}" AND
        authorEmail = "{email}"
"""

QUERY_SUCCESS_FACTOR = """
        SELECT
            *
        FROM
            `{BIGQUERY_SUCCESS_FACTOR}`
        WHERE
            email = "{email}"
    """

QUERY_SUCCESS_FACTOR_ALL = """
        SELECT
            *
        FROM
            `{BIGQUERY_SUCCESS_FACTOR}`
        WHERE
            office = '{office_name}' AND
            status = 'Active'
    """

QUERY_SUCCESS_FACTOR_ALL_DATA = """
        SELECT
            *
        FROM
            `{BIGQUERY_SUCCESS_FACTOR}`
    """

QUERY_EMPLOYEE_CAPACITY = """
        SELECT
            *
        FROM
            `{BIGQUERY_EMPLOYEE_CAPACITY}`
        WHERE
            email = "{email}" AND
            month='{month}' AND
            year='{year}'

    """

QUERY_ALL_EMPLOYEE_CAPACITY = """
        SELECT
            *
        FROM
            `{BIGQUERY_EMPLOYEE_CAPACITY}`
        WHERE
            month='{month}' AND
            year='{year}'
    """

QUERY_REGISTER_EMPLOYEE_CAPACITY = """
    INSERT INTO `{BIGQUERY_EMPLOYEE_CAPACITY}`
    (email, month, year, fte, office, jira, capacity, startdate, enddate, realcapacity) VALUES (
        '{email}',
        '{month}',
        '{year}',
        {fte},
        '{office}',
        {jira},
        {capacity},
        '{hiring_date}',
        {enddate},
        {realcapacity}
    )
    """

QUERY_UPDATE_EMPLOYEE_CAPACITY = """
    UPDATE `{BIGQUERY_EMPLOYEE_CAPACITY}` SET
        email='{email}',
        month='{month}',
        year='{year}',
        fte={fte},
        office='{office}',
        jira={jira},
        capacity={capacity},
        startdate='{hiring_date}',
        enddate='{enddate}'
    WHERE
        email='{email}' AND
        month='{month}' AND
        year='{year}'
    """

QUERY_HOURS_IMPUTED_OUT_DATE = """
    SELECT
        *
    FROM
        `{BIGQUERY_JIRA_DATA_TYPES}`
    WHERE
        created >= "{last_day}" AND
        started < "{first_day}"
"""

MS_FOR_HOUR = 3600000


def get_all_data(date, office_emails):
    from_date = get_first_day(date).strftime("%Y-%m-%d")
    to_date = get_last_day(date).strftime("%Y-%m-%d")
    email_query_list = ",".join(
        ["'{}'".format(email) for email in office_emails]
    )
    query = QUERY_HOURS_IMPUTED_OFFICE_EMPLOYEES.format(
        from_date=from_date,
        to_date=to_date,
        emails=email_query_list,
        BIGQUERY_JIRA_DATA_TYPES=BIGQUERY_JIRA_DATA_TYPES,
    )

    return execute_query(query, error_value=[])


def get_imputed_hours(from_date, to_date):
    from_date = from_date.strftime("%Y-%m-%d")
    to_date = to_date.strftime("%Y-%m-%d")
    query = QUERY_HOURS_IMPUTED_ALL_EMPLOYEES.format(
        from_date=from_date,
        to_date=to_date,
        BIGQUERY_JIRA_DATA_TYPES=BIGQUERY_JIRA_DATA_TYPES,
    )

    return execute_query(query, error_value=[])


def get_all_data_by_dates(from_date, to_date, office_emails):
    email_query_list = ",".join(
        ["'{}'".format(email) for email in office_emails]
    )
    query = QUERY_HOURS_IMPUTED_OFFICE_EMPLOYEES.format(
        from_date=from_date,
        to_date=to_date,
        emails=email_query_list,
        BIGQUERY_JIRA_DATA_TYPES=BIGQUERY_JIRA_DATA_TYPES,
    )

    return execute_query(query, error_value=[])


def get_data_from_email(email, date):
    from_date = get_first_day(date).strftime("%Y-%m-%d")
    to_date = get_last_day(date).strftime("%Y-%m-%d")
    query = QUERY_HOURS_IMPUTED.format(
        from_date=from_date,
        to_date=to_date,
        email=email,
        BIGQUERY_JIRA_DATA_TYPES=BIGQUERY_JIRA_DATA_TYPES,
    )

    return execute_query(query, error_value=[])


def get_successfactor_data(email):
    query = QUERY_SUCCESS_FACTOR.format(
        email=email, BIGQUERY_SUCCESS_FACTOR=BIGQUERY_SUCCESS_FACTOR
    )

    return execute_query(query, error_value=[])


def get_successfactor_all_data(office_name):
    query = QUERY_SUCCESS_FACTOR_ALL.format(
        BIGQUERY_SUCCESS_FACTOR=BIGQUERY_SUCCESS_FACTOR,
        office_name=office_name,
    )
    return execute_query(query, error_value=[])


def get_successfactor_all_offices():
    query = QUERY_SUCCESS_FACTOR_ALL_DATA.format(
        BIGQUERY_SUCCESS_FACTOR=BIGQUERY_SUCCESS_FACTOR
    )
    return execute_query(query, error_value=[])


def get_all_employee_capacity(month, year):
    query = QUERY_ALL_EMPLOYEE_CAPACITY.format(
        BIGQUERY_EMPLOYEE_CAPACITY=BIGQUERY_EMPLOYEE_CAPACITY,
        month=month,
        year=year,
    )
    return execute_query(query, error_value=[])


def get_employee_capacity(email, year, month):
    query = QUERY_EMPLOYEE_CAPACITY.format(
        email=email,
        year=year,
        month=month,
        BIGQUERY_EMPLOYEE_CAPACITY=BIGQUERY_EMPLOYEE_CAPACITY,
    )

    return execute_query(query, error_value=[])


def register_employee_capacity(
    email,
    year,
    month,
    hours,
    office,
    capacity,
    realcapacity,
    fte,
    hiring_date,
    enddate,
):  # noqa: E501
    insert_query_builder = BigqueryUpdateQueryBuilder(
        email,
        year,
        month,
        hours,
        fte,
        office,
        capacity,
        hiring_date,
        enddate,
        realcapacity,
    )
    query = insert_query_builder.build_insert()

    return execute_query(query, error_value=[])


def update_employee_capacity(
    email,
    year,
    month,
    hours,
    office,
    capacity,
    realcapacity,
    fte,
    hiring_date,
    enddate,
):  # noqa: E501
    update_query_builder = BigqueryUpdateQueryBuilder(
        email,
        year,
        month,
        hours,
        fte,
        office,
        capacity,
        hiring_date,
        enddate,
        realcapacity,
    )
    query = update_query_builder.build_update()

    return execute_query(query, error_value=[])


def create_or_update_employee(email, data):
    def _get_capacity_data(employee_capacity):
        capacity_data = {}
        for capacity in employee_capacity:
            capacity_data['fte'] = capacity.get('fte', 0.0)

        return capacity_data

    successfactor_data = data.pop('successfactor_data')
    employee_capacity = _get_capacity_data(
        get_employee_capacity(
            email, data['yearSelected'], data['monthSelected']
        )
    )
    totalTimeSpent = data['data']['totalTimeSpent'] * 1000

    if employee_capacity:
        update_employee_capacity(
            email,
            data['yearSelected'],
            data['monthSelected'],
            totalTimeSpent,
            successfactor_data['office_name'],
            data['agreementHours']['totalHours'] * MS_FOR_HOUR,
            data['agreementHours']['realcapacity'] * MS_FOR_HOUR,
            successfactor_data['FTE'],
            successfactor_data.get('hiring_date'),
            successfactor_data.get('enddate'),
        )
    else:
        register_employee_capacity(
            email,
            data['yearSelected'],
            data['monthSelected'],
            totalTimeSpent,
            successfactor_data['office_name'],
            data['agreementHours']['totalHours'] * MS_FOR_HOUR,
            data['agreementHours']['realcapacity'] * MS_FOR_HOUR,
            successfactor_data['FTE'],
            successfactor_data.get('hiring_date'),
            successfactor_data.get('enddate'),
        )


def create_batch_insert_values(email, data):
    def _get_date_value(date_value):
        if date_value:
            return f"'{date_value}'"
        else:
            return 'null'

    totalTimeSpent = data['data']['totalTimeSpent'] * 1000
    successfactor_data = data.pop('successfactor_data')
    enddate = _get_date_value(successfactor_data.get('enddate'))
    startdate = _get_date_value(successfactor_data.get("hiring_date"))

    return "('{email}','{yearSelected}','{monthSelected}',{totalTimeSpent},{fte},'{office}',{capacity},{realcapacity},{startdate},{enddate})".format(  # noqa: E501
        email=email,
        yearSelected=data["yearSelected"],
        monthSelected=data["monthSelected"],
        totalTimeSpent=totalTimeSpent,
        fte=successfactor_data["FTE"],
        office=successfactor_data["office_name"],
        capacity=data["agreementHours"]["totalHours"] * MS_FOR_HOUR,
        realcapacity=data["agreementHours"]["realcapacity"] * MS_FOR_HOUR,
        startdate=startdate,
        enddate=enddate,
    )


def insert_batch_capacities(capacities):
    query = f"INSERT INTO `{BIGQUERY_EMPLOYEE_CAPACITY}` (email, year, month, jira, fte, office, capacity, realcapacity, startdate, enddate) VALUES "  # noqa: E501
    values = ",".join(capacities)

    return execute_query(query + values)


def remove_data(year, month, office):
    query = f"DELETE FROM `{BIGQUERY_EMPLOYEE_CAPACITY}` WHERE year='{year}' AND month='{month}' AND office='{office}'"  # noqa: E501

    return execute_query(query, error_value=[])


def get_out_date_imputations(first_day, last_day):
    query = QUERY_HOURS_IMPUTED_OUT_DATE.format(
        first_day=first_day,
        last_day=last_day,
        BIGQUERY_JIRA_DATA_TYPES=BIGQUERY_JIRA_DATA_TYPES,
    )

    return execute_query(query, error_value=[])



from datetime import datetime

HUNDRED_PERCENT = 100
DEFAULT_FTE = 1.0  # 100 %


def get_real_fte(successfactor, employeeFTE, date):
    """Calculate employee FTE.
    When we want to calculate the FTE of an employee, we do it for a specific
    month since this can change.

     Rules:
         - If the date we want to calculate the FTE is from the current
           month into the future, then we will take the FTE value returned
           by the Success factor Bigquery table.
         - If for the month we want to calculate the FTE is prior to
           the current one and it already has a value in the Capacity
           table, this means that it was already calculated and we will
           use that value.

     args:
        successfactor (dict): BigQuery results from the successfactor table.
        employeeFTE (dict): Bigquery results from the Capacity table.
        date (date): Month on which the FTE is being calculated

    Returns:
        float: Real FTE for a employee in specific month.
    """
    today = datetime.today()
    is_past_date = (date.year, date.month) <= (today.year, today.month)

    try:
        # Some employees has null as FTE.
        fte = successfactor['FTE'] / HUNDRED_PERCENT
    except (KeyError, TypeError):
        fte = DEFAULT_FTE

    if employeeFTE and is_past_date:
        fte = employeeFTE['fte']

    return fte


def calculate_real_capacity(
    agreement_hours,
    hiringdate,
    end_date,
    date,
    to_date,
    fte,
    calculate_range=False,
):
    def _is_date_before_hired():
        return hiringdate and (
            (date.year, date.month) < (hiringdate.year, hiringdate.month)
        )

    def _is_date_same_hired():
        return hiringdate and (
            (date.year, date.month) == (hiringdate.year, hiringdate.month)
        )

    def _is_date_between_hired_and_end_date():
        is_date_before_end_date = not end_date or (
            (date.year, date.month) < (end_date.year, end_date.month)
        )
        return (
            is_date_before_end_date
            and hiringdate
            and (
                (date.year, date.month) >= (hiringdate.year, hiringdate.month)
            )
        )

    def _is_date_same_end_date():
        return end_date and (
            (date.year, date.month) == (end_date.year, end_date.month)
        )

    def _is_date_after_end_date():
        return end_date and (
            (date.year, date.month) > (end_date.year, end_date.month)
        )

    real_capacity = agreement_hours.get('totalHours', 0)

    if _is_date_before_hired():
        # Months prior to hiring the employee
        real_capacity = 0
    elif calculate_range and _is_date_between_hired_and_end_date():
        # Just when we want to calculate hours by range of date
        real_capacity = 0
        days = agreement_hours.get('days', {})
        for dayMonth, hours in days.items():
            day, month = dayMonth.split('-')
            is_date_after_from = (int(day), int(month)) >= (
                date.day,
                date.month,
            )
            is_date_before_to = (int(day), int(month)) <= (
                to_date.day,
                to_date.month,
            )
            if is_date_after_from and is_date_before_to:
                real_capacity += float(hours.replace(',', '.'))
    elif _is_date_same_hired():
        # Same month of hiring the employee
        real_capacity = 0
        days = agreement_hours.get('days', {})

        for day, hours in days.items():
            if int(day) >= hiringdate.day:
                real_capacity += float(hours.replace(',', '.'))

    if _is_date_same_end_date():
        # Same month of the termination of the employment contract
        real_capacity = 0
        days = agreement_hours.get('days', {})
        for day, hours in days.items():
            if int(day) <= end_date.day:
                real_capacity += float(hours.replace(',', '.'))

    elif _is_date_after_end_date():
        # Months after the termination of the employee's contract
        real_capacity = 0

    return real_capacity * fte



import calendar
import datetime

from dateutil.relativedelta import relativedelta


def get_first_day(date):
    return datetime.date(date.year, date.month, 1)


def get_last_day(date):
    return datetime.date(
        date.year, date.month, calendar.monthrange(date.year, date.month)[1]
    )


def subtract_months_for_date(date, months_to_subtract=0):
    return (date - relativedelta(months=months_to_subtract)).date()
